function [xEst,ZMean,G]=KalmanSkalarX_Z(F,B,Q,R,H,u,z,xTrue)

%    [xEst,ZMean,K]=KalmanSkalarX_Z(F,B,Q,H,R,u,z,xTrue)
%A becslshez szuksges "valdi" szimullt (skalr) jelsorozatot - xTrue, a
%rendszerre vonatkoz tudsunk (F,B,Q,H,R,u), illetve a mrsi sorozat (z)
%alapjn el?lltja a Klmn szuro alapu becslst xTrue-nak (xEst).
%Visszaadja tovbb a mrt jelek egyszer? tlagaknt nyert ZMean becslst, s a Klmn
%er?sts alakulst (G).
%
%Az rzkel?k szmt  a H megfigyelsi vektor hatrozza meg size(H)= [NS,1].
%A mrsi sorozat hosszt az 'u' bemeneti jel hossza hatrozza meg (N).
%Az rzkelo jelsorozat mrete ennek megfelel?en size(z)=[NS,N] kell legyen. 
% Az llapotegyenlet:
%     x(k)=F*x(k-1)+B*u(k)+v;
% ahol v egy nulla vrhat rtk? Q szrs fehrzaj.
% s a megfigyelsi egyenlet:
%     z(k)=H*x(k)+w(k);
% ahol H hatrozza meg, hogy hny szenzorunk van (size(H)=[NSz,1])) ,
% w(k) pedig egy NS dimenzis nulla vrhatrtk? Gauss mrsi zaj, a koveriancia mtrixa
% diagonl (a zajok egymssal nem korrelltak, mrete: size(R)=[NS,NS].


%% mrsi hossz (N), szenzorszm (NS) paramterek + bemeneti adatok
% mretellenorzse
if min(size(u))~=1, disp('a bemeneti vektor (u) helyett mtrix van megadva!'); return, end
N=max(size(u));   % a bemeneti jel hossza N, amintk szma
if N<10, disp('a bemeneti vektor tl rvid (N<10)!');return,end 
if size(H,2)~=1, disp('a megfigyelsi vektor oszlopvektor kell legyen!'); return, end
%A szenzorok szma
NS=size(H,1);

%% az llapotegyenlet (megadott paramterek F,B,Q mretellen?rzse)

% x(k)=F*x(k-1)+B*u(k)+v;
% v variancijt  Q adja

if not(isscalar(F)) || not(isscalar(B))  || not(isscalar(Q)),
    disp('az llapotegyenlet mindhrom paramtere (F,B,Q) skalr kell legyen!');
    return,
end

%%inicalizls
% KPred, KEst, xPred, xEst skalr
% a Klmn er?sts minden id?pilanatban egy [NS,1] oszlopvektor
KPred=zeros(1,N);  % Az llapotvltozo kovariancijnak josolt K(k|k-1) becslse
KEst=zeros(1,N);   % Az llapotvltozo kovariancijnak K(k|k) becslse

xPred=zeros(1,N);
xEst=zeros(1,N);

G=zeros(NS,N);

%% a Kalman szuro algoritmus
for k=2:N
    xPred(1,k)=F*xEst(1,k-1)+B*u(k);
    KPred(1,k)=F*KEst(1,k-1)*F'+Q;
    G(:,k)=KPred(1,k)*H'*inv(H*KPred(1,k)*H'+R);
    xEst(k)=xPred(1,k)+G(:,k)'*(z(:,k)-H*xPred(1,k));
    KEst(1,k)=(1-G(:,k)'*H)*KPred(1,k);
end


%%Megjelents
tt=1:N;
ZMean=mean(z);
figure(1),plot(tt,xEst,'b',tt,z(1,:),'g',tt,ZMean,'m',tt,xTrue,'r')
title('xEst - kk, z(1,:) - zld, a 3 mrs tlaga (ZMean) - magenta, xTrue - piros')

figure(2),plot(tt,G(1,:))
title('A Kalman erosites az elso mrsi komponensre')

figure(3),plot(tt,KEst)
title('az llapotbecsls kovariancijnak becslse')